@@ -33,6 +33,9 @@ class Redis_Cluster_Test extends Redis_Test {
     public function testSelect()   { return $this->markTestSkipped(); }
     public function testReconnectSelect() { return $this->markTestSkipped(); }
     public function testMultipleConnect() { return $this->markTestSkipped(); }
+    public function testDoublePipeNoOp() { return $this->markTestSkipped(); }
+    public function testSwapDB() { return $this->markTestSkipped(); }
+    public function testConnectException() { return $this->markTestSkipped(); }
 
     /* Load our seeds on construction */
     public function __construct() {
